import processing.core.*; 
import processing.xml.*; 

import ddf.minim.*; 
import ddf.minim.analysis.*; 
import ddf.minim.effects.*; 
import ddf.minim.ugens.*; 
import ddf.minim.spi.*; 
import net.compartmental.contraptions.*; 
import processing.opengl.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class boombox extends PApplet {

/** <br/>
 * Use WASD to move around (or the mouse). <br/>
 * Pick up Jams (the cassettes) and Effect (the pedals) by walking over them.<br/>
 * Click on a Jam in your inventory at the top of the screen to play it or eject it.<br/>
 * Click on an Effect in your inventory to apply it to your mix.<br/>
 * Stand next to a green dude and play what he wants to hear to jam with him.<br/>
 * Mix and match your Jams and Effects to make a Fresh Tune.<br/>
 */











boolean DRAW_COLLISION = false;
int   COLLISION_COLOR = color(0, 255, 0, 128);

Thread loaderThread;
TitleScreen titleScreen;
GameplayScreen gameplayScreen;

public void setup()
{
  size(640, 480, OPENGL);
  
  loaderThread = new Thread( new AssetLoader(this) );
  loaderThread.start();
  
  titleScreen = new TitleScreen();
  gameplayScreen = new GameplayScreen();
}

public void draw()
{ 
  float dt = 1.f / frameRate;
  
  if ( titleScreen.isActive() )
  {
    titleScreen.draw();
  }
  else
  {
    gameplayScreen.update( dt );
    gameplayScreen.draw();
  }
}

public void mousePressed()
{
  if ( titleScreen.isActive() )
  {
    titleScreen.mousePressed();
  }
  else
  {
    gameplayScreen.mousePressed();
  }
}

public void mouseMoved()
{
  if ( titleScreen.isActive() == false )
  {
    gameplayScreen.mouseMoved();
  }
}

public void mouseReleased()
{
  if ( titleScreen.isActive() == false )
  {
    gameplayScreen.mouseReleased();
  }
}

boolean filterOn = false;

public void keyPressed()
{
  if ( titleScreen.isActive() == false )
  {
    gameplayScreen.keyPressed();
  }
}

public void keyReleased()
{
  if ( titleScreen.isActive() == false )
  {
    gameplayScreen.keyReleased();
  }
}

// audio junk
Minim minim;

// our main audio path
JamSyncer jamSyncer;
SampleAndRepeat sampleRepeat;
Bypass granulateBypass;
GranulateSteady granulate;
ChebFilter lowPass;
Gain  globalGain;
AudioOutput mainOut;

// controls
Line lowPassCutoff;
float lowPassCutoffSweepLength; // set in setup()
float lowPassCutoffHi = 21000.f;
float lowPassCutoffLo = 400.f;

// something to plug envelope followers into
Sink envFollowSink;

// SFX
AudioSample tapeGet;
AudioSample tapePlay;
AudioSample tapeStop;

// visual junks
Avatar player;
Mouse  mouse;
Inventory inventory;
Stage     theStage;

AnimationSystem animationSystem;

// all the Jams in the game
// Jam 0 is the backing jam
// Jam 1 is the first cassette you pick up
ArrayList<Jam> allJams;
// jams that are out in the world
// just, like, minding their own business, yo
ArrayList<Jam> worldJams;
ArrayList<EffectPickup> worldEffects;

class AssetLoader implements Runnable
{
  PApplet m_parent;
  
  AssetLoader( PApplet parent )
  {
    m_parent = parent;
  }
  
  public void run()
  {
      animationSystem = new AnimationSystem(m_parent);
      animationSystem.loadAnimations( "animation/animations.xml" );

      loadEffectImage();
      
      TAPE = loadImage("BoomBox_Cassette_No_Teeth.png");
      
      loadSpeakerImages();
      loadStarImages();
      
      float tempo = 121.f;
      float beatPerSec = 60.f / tempo;
    
      minim = new Minim(m_parent);
      mainOut = minim.getLineOut( Minim.STEREO, 512 );
      jamSyncer = new JamSyncer( tempo );
      sampleRepeat = new SampleAndRepeat( tempo, 0.5f );
      float grainRatio = 0.03f;
      granulate = new GranulateSteady( beatPerSec * (0.125f - grainRatio), beatPerSec * (0.125f + grainRatio), 0.0005f, // grainLength, spaceLength, fadeLength
                                       0.f, 1.f // min and max amplitude
                                     );
      granulateBypass = new Bypass( granulate );
      granulateBypass.activate();
      lowPass = new ChebFilter( lowPassCutoffHi, ChebFilter.LP, 2.0f, 4, mainOut.sampleRate() );
      globalGain = new Gain(0.f);
    
      lowPassCutoffSweepLength = (60.f / 121.f) * 4;
      lowPassCutoff = new Line( lowPassCutoffSweepLength, lowPassCutoffHi, lowPassCutoffLo );
      lowPassCutoff.patch( lowPass.cutoff );
    
      envFollowSink = new Sink();
      envFollowSink.patch( mainOut );
    
      tapeGet  = minim.loadSample( "tape_on.wav" );
      tapeGet.setGain( 2.f );
      tapePlay = minim.loadSample( "tape_in.wav" );
      tapePlay.setGain( 2.f );
      tapeStop = minim.loadSample( "tape_stop.wav" );
      tapeStop.setGain( 2.f );
    
      allJams = new ArrayList<Jam>();
    
      allJams.add( new Jam("backing_loop.wav", JamCategory.OTHER, color(0,0,0), -100, -100 ) );
      allJams.add( new Jam("drums_LP01.wav",   JamCategory.DRUMS, 0xff01e4ff,      580, 120) );
      allJams.add( new Jam("bass_LP01.wav",    JamCategory.BASS,  0xff77da6f,      600, 400) );
      allJams.add( new Jam("blip_LP01.wav",    JamCategory.BLIP,  0xffffffff,      200, 600) );
      allJams.add( new Jam("pad_LP04.wav",     JamCategory.PAD,   0xffa966cf,      100, 450) );
      allJams.add( new Jam("drums_LP02.wav",   JamCategory.DRUMS, 0xff28a0ae,      580, 120) );
      allJams.add( new Jam("chords_LP02.wav",  JamCategory.CHORD, 0xff212bc0,      600, 400) );
      allJams.add( new Jam("blip_LP02.wav",    JamCategory.BLIP,  0xffffee33,      200, 600) );
      allJams.add( new Jam("pad_LP02.wav",     JamCategory.PAD,   0xff292928,      100, 450) );
      allJams.add( new Jam("drums_LP03.wav",   JamCategory.DRUMS, 0xffd93c65,      580, 120) );
      allJams.add( new Jam("bass_LP03.wav",    JamCategory.BASS,  0xfffe867d,      600, 400) );
      allJams.add( new Jam("blip_LP03.wav",    JamCategory.BLIP,  0xffe7a858,      200, 600) );
      allJams.add( new Jam("blip_LP05.wav",    JamCategory.BLIP,  0xfff4be2e,      100, 450) );
    
      worldJams = new ArrayList<Jam>();
    
      worldJams.add( allJams.get(1) );
      worldJams.add( allJams.get(3) );
      worldJams.add( allJams.get(5) );
      worldJams.add( allJams.get(8) );
      worldJams.add( allJams.get(9) );
      worldJams.add( allJams.get(11));
    
      worldEffects = new ArrayList<EffectPickup>();
    
      worldEffects.add( new EffectPickup( new LineSweep( lowPassCutoff, lowPassCutoffHi, lowPassCutoffLo, lowPassCutoffSweepLength ), 800, 450, 0xffafa38b ) );
      worldEffects.add( new EffectPickup( new Bypasser( granulateBypass ), 1350, 400, 0xffffffff ) );
      worldEffects.add( new EffectPickup( new SampleRepeat( sampleRepeat ), 3150, 410, 0xff67606b ) );
    
      player = new Avatar( 50, height - 100 );
    
      mouse = new Mouse();
      inventory = new Inventory();
      theStage = new Stage();
  }
}
// the thing the player controls
class Avatar
{
  private PVector mPos; // where I am
  private PVector mGoal; // where I am going
  private float   mSpeed = 175.f;
  private int     mXDir = 0;
  private int     mYDir = 0;   
  private float   mScale = 1.f;
  private AnimationStateMachine mAnims;

  Avatar( float xPos, float yPos )
  {
    mPos = new PVector( xPos, yPos );
    mGoal = new PVector( mPos.x, mPos.y );
    mAnims = new AnimationStateMachine( animationSystem, new XMLElement( boombox.this, "animation/player.xml" ) );
  }

  public void setXDir( int dir )
  {
    if ( mXDir != dir )
    { 
      mXDir = dir;

      if ( mXDir < 0 )
      {
        mScale = -1.f;
      }
      else if ( mXDir > 0 )
      {
        mScale = 1.f;
      }
    }
  }

  public void setYDir( int dir )
  {
    if ( mYDir != dir )
    { 
      mYDir = dir;
    }
  }

  public void setGoal( float x, float y )
  {
    mGoal.set( x, y, 0 );
  }

  public void clearGoal()
  {
    mGoal.set( mPos.x, mPos.y, 0 );
  }

  public void setYPos( float y )
  {
    mPos.y = y;
  }

  public PVector getPos()
  {
    return new PVector( mPos.x, mPos.y );
  }

  public Rectangle getCollisionRectangle()
  {
    return new Rectangle( mPos.x, mPos.y, getWidth() * 0.8f, getHeight() * 0.8f, CENTER, BOTTOM );
  }

  public float getWidth()
  {
    return mAnims.currentAnimation().width();
  }

  public float getHeight()
  {
    return mAnims.currentAnimation().height();
  }

  public void update( float dt )
  { 
    String currentAnim = mAnims.getCurrentStateName();

    if ( currentAnim.equals( "idle" ) || currentAnim.equals( "walk" ) )
    {
      PVector dir = new PVector( mXDir, mYDir );
      dir.normalize();
      dir.mult( mSpeed * dt );

      // where we hope to wind up
      PVector newPos = new PVector( mPos.x + dir.x, mPos.y + dir.y );

      theStage.constrainMovement( mPos, newPos );

      if ( mPos.x == newPos.x && mPos.y == newPos.y )
      {
        mAnims.sendEvent( "idle" );
      }
      else
      {
        mAnims.sendEvent( "walk" );
      }

      mPos.set( newPos );
    }

    mAnims.update( dt );
  }

  public void draw()
  {
    pushMatrix();
    {
      imageMode( CENTER ); 
      tint(255);
      translate( mPos.x, mPos.y  - mAnims.currentAnimation().height() / 2.f );
      scale( mScale, 1 );
      mAnims.draw();

      // reflection?
      //      tint(255, 24);
      //      translate( 0, mAnims.currentAnimation().height() - 2);
      //      scale( 1, -1 );
      //      mAnims.draw();
    }
    popMatrix();
  }

  public void collect()
  {
    // println("collect");
    tapeGet.trigger();
    mAnims.sendEvent( "collect" );
  }

  public void play()
  {
    mAnims.sendEvent( "play" );
  }

  public void eject()
  {
    mAnims.sendEvent( "eject" );
  }

  public void jam()
  {
    // println("Player going to Jam!");
    mAnims.sendEvent( "jam" );
  }

  public boolean isJamming()
  {
    return mAnims.getCurrentStateName().equals( "jam" );
  }

  public void idle()
  {
    mAnims.sendEvent( "idle" );
  }
}

// a Dude is someone who you can Jam with
class Dude implements LoopListener, AnimationStateMachine.EventListener
{
  // our visible rectangle
  private Rectangle mRect;
  // have to be inside this one to jam
  private Rectangle mJamArea;
  // the jams this dude wants to jam with
  private Jam[]   mWantToHear;
  // the jam this dude will play and then give away
  private Jam     mMyJam;
  // have I already jammed?
  boolean         mJammed;
  // how many times my jam has looped
  private int     mLoops;
  
  // is the player close enough to jam?
  boolean         mCanJam;
  // did I give my jam away
  boolean         mGaveJam;
  
  private PVector mPos;
  
  private AnimationStateMachine mAnims;
  
  // different animations
  static final int IDLE = 0;
  static final int SHAKE = 1;
  static final int JUMP = 2;
  
  Dude( float x, float y, Jam[] wantToHear, Jam toPlay )
  {
    mPos = new PVector(x, y);
    mRect = new Rectangle( x, y, 64, 64, CENTER, BOTTOM );
    mJamArea = new Rectangle( x, y, 256, 64, CENTER, CENTER );
    mWantToHear = wantToHear;
    mMyJam = toPlay;
    
    mJammed = false;
    mGaveJam = false;
    mCanJam = false;
    mLoops = 0;
    
    mAnims = new AnimationStateMachine( animationSystem, new XMLElement(boombox.this, "animation/dude.xml") );
    mAnims.addEventListener( this );
  }
  
  public PVector getPos()
  {
    return mRect.getPos();
  }
  
  public void ejectJam()
  {
    mAnims.sendEvent( "eject" );
    mMyJam.queue();
    mLoops = 0;
    jamSyncer.removeLoopListener( this ); 
    
    if ( player.isJamming() )
    {
      player.idle();
    }
  }
  
  public void update( float dt )
  {
    if ( mGaveJam == false )
    {
        mMyJam.update( dt );
        
        boolean canHear = mJamArea.pointInside( player.getPos() );
        
        if ( canHear )
        {
          boolean wantToJam = wantToJam();
        
          if ( wantToJam && mMyJam.isPlaying() == false && mMyJam.willPlay() == false && mMyJam.willEject() == false )
          {
            mAnims.sendEvent( "yesjam" );
          }
          else if ( wantToJam == false )
          {
            if ( (mMyJam.isPlaying() || mMyJam.willPlay()) && mMyJam.willEject() == false )
            {
              ejectJam();
            }
            else
            {
              mAnims.sendEvent( "nojam" );
            }
          }
        }
        else
        {
          if ( mMyJam.willPlay() )
          {
            ejectJam();
          }
          else if ( mAnims.getCurrentStateName().equals( "eject" ) == false )
          {
            mAnims.sendEvent( "idle" );
          }
        }
    }
    else
    {   
      if ( mMyJam.isPlaying() )
      {
        mAnims.sendEvent( "jam" );
      }
      else if ( mMyJam.willPlay() == false && mMyJam.willEject() == false )
      {
        mAnims.sendEvent( "idle" );
      }
    }
    
    mAnims.update( dt );
  }
  
  public boolean wantToJam()
  {
      for(int i = 0; i < mWantToHear.length; ++i)
      {
        // bail if the player isn't playing something I want to hear
        if ( (mWantToHear[i].isPlaying() == false && mWantToHear[i].willPlay() == false) || mWantToHear[i].willEject() )
        {
          // println("Something I want to hear isn't playing!");
          return false;
        }
      }
      
      return true;
  }
  
  public void looped()
  {
    mLoops++;
    if ( mLoops == 1 )
    {
      mAnims.sendEvent( "jam" );
      player.jam();
    }
    else if ( mLoops == 2 )
    {
      inventory.addJam( mMyJam );
      mGaveJam = true;
      player.collect();
      jamSyncer.removeLoopListener( this );
      
      // spawn a star!
      gameplayScreen.addStar( mPos.x, mPos.y - 100 );
    }
  }
  
  public void eventSent( AnimationStateMachine.Event event )
  {
    if ( event.getName().equals("play") )
    {
      mMyJam.queue();
      jamSyncer.addLoopListener( this ); 
    }
  }
  
  public void draw()
  { 
    pushMatrix();
    {
      tint(0, 255, 255);
      imageMode(CENTER);
      translate( mPos.x, mPos.y - mAnims.currentAnimation().height() / 2.f );
      if ( player.getPos().x < mPos.x )
      {
        scale( -1, 1 );
      }
      mAnims.draw();
    }
    popMatrix();
    
//    fill(0, 64);
//    mJamArea.draw();
    
    if ( mGaveJam == false )
    {
      PVector pos = mRect.getPos();
      
      rectMode(CENTER);
      
      stroke(0);
      strokeWeight(2);
      fill(255);
      rect( pos.x + 5, pos.y - 55, 5, 5 );
      rect( pos.x + 10, pos.y - 68, 10, 10 );
      
      float bubbleWidth = getTapeWidth() * mWantToHear.length + 5 * mWantToHear.length + 10;
      float bubbleHeight = getTapeHeight() + 30;
      float bubbleCenterY = pos.y - 105;
      rect( pos.x, bubbleCenterY, bubbleWidth, bubbleHeight );
      
      // these are the tapes in the bubble
      if ( mMyJam.isPlaying() == false && mMyJam.willPlay() == false )
      {
        float startX = pos.x - (bubbleWidth*0.5f) + (getTapeWidth()*0.5f) + 8;
        for(int i = 0; i < mWantToHear.length; i++)
        {
          JamImage img = mWantToHear[i].getImage();
          float offsetX = i * (getTapeWidth()+5);
          img.draw( startX + offsetX, bubbleCenterY, 1.f );
        }
      }
      else
      {
        mMyJam.setPos( pos.x, bubbleCenterY );
        if ( mMyJam.isPlaying() )
        {
          mMyJam.drawOnlyImage();
        }
        else
        {
          mMyJam.draw();
        }
      }
    }
  }
  
}
// you can pick up effects, just like jams, but they make the audio sound different, instead of adding new audio.

PImage EFFECT = null;
PImage EFFECT_ON = null;

public void loadEffectImage()
{
  EFFECT = loadImage("Pedal.png");
  EFFECT_ON = loadImage("Pedal_RedLight.png");
}

interface Effect 
{
  public void activate();
  public void deactivate();
}

class EffectPickup
{
  private PVector mPos;
  private float   mWidth, mHeight, mScale;
  private int   mColor;
  private boolean mIsActive;
  private Effect  mEffect;
  
  EffectPickup( Effect e, float x, float y, int c )
  {
    mPos = new PVector(x, y);
    mWidth = EFFECT.width;
    mHeight = EFFECT.height;
    mScale = 1f;
    mColor = c;
    mIsActive = false;
    mEffect = e;
  }
  
  public boolean isActive()
  {
    return mIsActive;
  }
  
  public void activate()
  {
    if ( mIsActive == false )
    {
      mIsActive = true;
      mEffect.activate();
    }
  }
  
  public void deactivate()
  {
    if ( mIsActive == true )
    {
      mIsActive = false;
      mEffect.deactivate();
    }
  }
  
  public void setPos( float x, float y )
  {
    mPos.set( x, y, 0 );
  }

  public PVector getPos()
  {
    return mPos;
  }
  
  public void setScale( float s )
  {
    mScale = s;
  }
  
  public Rectangle getCollisionRectangle()
  {
    return new Rectangle( mPos.x, mPos.y, getWidth(), getHeight(), CENTER, CENTER );
  }

  public float getWidth()
  {
    return mWidth * mScale;
  }
  
  public float getHeight()
  {
    return mHeight * mScale;
  }
  
  public boolean pointInside( float x, float y )
  {
    float hw = getWidth() * 0.5f;
    float hh = getHeight() * 0.5f;
    return ( x > mPos.x - hw && x < mPos.x + hw && y > mPos.y - hh && y < mPos.y + hh );
  }
  
  public void draw()
  {
    imageMode( CENTER );
    tint( mColor );
    image( EFFECT, mPos.x, mPos.y, getWidth(), getHeight() );
    
    if ( isActive() )
    {
      noTint();
      image( EFFECT_ON, mPos.x, mPos.y, getWidth(), getHeight() );

//      stroke(255);
//      strokeWeight(2);
//      fill(0);
//      rectMode( CENTER );
//      rect( mPos.x, mPos.y, 10, 10 );
    }
  }
}

class LineSweep implements Effect
{
  private Line mLine;
  private float mHi, mLo, mLength;
  
  LineSweep( Line line, float hi, float lo, float len )
  {
    mLine = line;
    mHi = hi;
    mLo = lo;
    mLength = len;
  }
  
  public void activate()
  {
    mLine.activate( mLength, mLine.getLastValues()[0], mLo );
  }
  
  public void deactivate()
  {
    mLine.activate( 0.2f, mLine.getLastValues()[0], mHi );
  }
}

class Bypasser implements Effect
{
  private Bypass mBypass;
  
  Bypasser( Bypass b )
  {
    mBypass = b;
  }
  
  public void activate()
  {
    mBypass.deactivate();
  }
  
  public void deactivate()
  {
    mBypass.activate();
  }
}

class SampleRepeat implements Effect
{
  private SampleAndRepeat mSample;
  
  SampleRepeat( SampleAndRepeat sample )
  {
    mSample = sample;
  }
  
  public void activate()
  {
    mSample.activate();
  }
  
  public void deactivate()
  {
    mSample.deactivate();
  }
}
// an Elevator is an audio-reactive platform
class Elevator
{
  private Rectangle mRect;
  private float     mX, mMinY, mMaxY;
  private EnvelopeFollower mFollower;
  private int mColor;
  
  Elevator( float x, float y, float maxVertDisp, Jam jamToFollow )
  {
    mRect = new Rectangle( x, y, 100, 10, CENTER, TOP );
    mX = x;
    mMaxY = y;
    mMinY = mMaxY - maxVertDisp;
    mColor = jamToFollow.getColor();
    mFollower = new EnvelopeFollower( 0.05f, 12.0f, 512 ); // attack, release, buffersize
    jamToFollow.patch( mFollower ).patch( envFollowSink );
  }
  
  public Rectangle getRect()
  {
    return mRect;
  }
  
  public void update( float dt )
  {
    float y = constrain( mMaxY - mFollower.getLastValues()[0] * 10000.f, mMinY, mMaxY );
    
    PVector playerPos = player.getPos();
    if ( mRect.pointInside( playerPos ) )
    {
      float deltaY = y - mRect.getPos().y;
      player.setYPos( playerPos.y + deltaY );
    }
    
    mRect.setPos( mX, y );
  }
  
  public void draw()
  { 
    rectMode(CORNER);
    Rectangle.Bounds bounds = mRect.getBounds();
//    stroke(0);
//    strokeWeight(2);
//    rect( bounds.minX, bounds.minY, bounds.maxX, bounds.maxY + 10 );

    noStroke();
    float totalWidth = bounds.maxX - bounds.minX;
    float totalHeight = bounds.maxY - bounds.minY;
    float barWidth = (totalWidth - 12) * 0.25f;
    float spacer = 12 / 3;
    for(int i = 0; i < 4; ++i)
    {
      float xOff = i * (barWidth + spacer);
      
      fill(mColor);
      rect( bounds.minX + xOff, bounds.minY, barWidth, totalHeight );
      
      int dark = 34;
      int alph = 128;
      float minX = bounds.minX + xOff;
      float maxX = minX + barWidth;
      float minY = bounds.maxY;
      float maxY = mMaxY + totalHeight;
      beginShape( QUADS );
        fill( mColor, alph );
        vertex( minX, minY );
        fill( mColor, 0 );
        vertex( minX, maxY );
        vertex( maxX, maxY );
        fill( mColor, alph );
        vertex( maxX, minY );
      endShape();
        
      //rect( bounds.minX + xOff, bounds.maxY, barWidth, mMaxY + totalHeight - bounds.maxY );
    }
    
    // our "collision"
    if ( DRAW_COLLISION )
    {
      noStroke();
      fill(COLLISION_COLOR);
      mRect.draw();
    }
  }
}
class GameplayScreen
{
  // where my camera at
  float cameraPosition = 0;

  // what is my camera's offset from where it started?
  float cameraOffset = 0;
  
  ArrayList<Star> mStars;
  
  GameplayScreen()
  {
      cameraPosition = width/2;
      mStars = new ArrayList<Star>();
  }
  
  public void addStar(float x, float y)
  {
    mStars.add( new Star(x, y) );
  }
  
  public void update( float dt )
  {
    noCursor();

    if ( inventory.isActive() == false && mousePressed )
    {
      doMouseMove();
    }
  
    theStage.update( dt );
    player.update( dt );
    inventory.update( dt );
    
    Iterator<Star> iter = mStars.iterator();
    while( iter.hasNext() )
    {
      if ( iter.next().update(dt) )
      {
        iter.remove();
      }
    }
  
    float playerX = player.getPos().x;
    float distToCamera = abs( playerX - cameraPosition );
    float cameraSpeed = 175;
    if ( distToCamera < 100 )
    {
      if ( playerX < cameraPosition )
      {
        cameraPosition += cameraSpeed * dt;
      }
      else
      {
        cameraPosition -= cameraSpeed * dt;
      }
    }
    else if ( distToCamera > 200 )
    {
      if ( playerX < cameraPosition )
      {
        cameraPosition -= cameraSpeed * dt;
      }
      else
      {
        cameraPosition += cameraSpeed * dt;
      }
    }
  
    cameraPosition = constrain( cameraPosition, width/2, theStage.moreGround.getBounds().maxX - width/2 );
    cameraOffset = cameraPosition - width/2;
  }
  
  public void draw()
  {
    float lowPassCut = lowPass.frequency();
    float r = map( lowPassCut, lowPassCutoffHi, lowPassCutoffLo, 24, 0 );
    float g = r;
    float b = map( lowPassCut, lowPassCutoffHi, lowPassCutoffLo, 24, 60 );
    
    background( r, g, b );
    
    //  camara viz
    //  fill(255,0,0);
    //  noStroke();
    //  rectMode(CENTER);
    //  rect( cameraPosition, height/2.0, 5, 5 );
  
    camera( cameraPosition, height/2.0f, (height/2.0f) / tan(PI*60.0f / 360.0f), cameraPosition, height/2.0f, 0, 0, 1, 0);
  
    if ( false )
    {
      fill(0, 64);
      rectMode(CORNER);
      rect( cameraOffset - 5, 0, width + 5, height );
    }
  
    // will also draw the player sorted properly with stage elements
    theStage.draw();
  
    Iterator<Jam> iter = worldJams.iterator();
    while ( iter.hasNext() )
    {
      Jam j = iter.next();
  
      if ( player.getCollisionRectangle().collidesWith( j.getCollisionRectangle() ) )
      {
        iter.remove();
        inventory.addJam( j );
        player.collect();
      }
      else
      {
        j.draw();
      }
    }
  
    Iterator<EffectPickup> eiter = worldEffects.iterator();
    while( eiter.hasNext() )
    {
      EffectPickup e = eiter.next();
  
      if ( player.getCollisionRectangle().collidesWith( e.getCollisionRectangle() ) )
      {
        eiter.remove();
        inventory.addEffect( e );
        player.collect();
      }
      else
      {
        e.draw();
      }
    }
    
    Iterator<Star> siter = mStars.iterator();
    while( siter.hasNext() )
    {
      siter.next().draw();
    }
  
    translate( cameraPosition - width/2, 0, 0 );
  
    // jams we can play  
    inventory.draw();
  
    // mouse appearance
    mouse.draw();
  }
  
  public void mousePressed()
  {
    inventory.mousePressed();
  }
  
  public void mouseReleased()
  {
    player.setXDir( 0 );
    player.setYDir( 0 );
  }
  
  public void mouseMoved()
  {
    inventory.mouseMoved();
  }
  
  public void doMouseMove()
  {
    float mouseX = boombox.this.mouseX;
    float mouseY = boombox.this.mouseY;
    
    PVector pos = player.getPos();
    if ( cameraOffset + mouseX < pos.x - player.getWidth() * 0.4f )
    {
      player.setXDir( -1 );
    }
    else if ( cameraOffset + mouseX > pos.x + player.getWidth() * 0.4f )
    {
      player.setXDir( 1 );
    }
    else
    {
      player.setXDir( 0 );
    }
  
    if ( mouseY < pos.y - player.getHeight() )
    {
      player.setYDir( -1 );
    }
    else if ( mouseY > pos.y )
    {
      player.setYDir( 1 );
    }
    else
    {
      player.setYDir( 0 );
    }
  }
  
  public void keyPressed()
  {
    char key = boombox.this.key;
    if ( key != PApplet.CODED )
    {
      if ( key == 'd' || key == 'D' )
      {
        player.setXDir( 1 );
      }
      else if ( key == 'a' || key == 'A' )
      {
        player.setXDir( -1 );
      }
      else if ( key == 'w' || key == 'W' )
      {
        player.setYDir( -1 );
      }
      else if ( key == 's' || key == 'S' )
      {
        player.setYDir( 1 );
      }
      else if ( (key == 'r' || key == 'R')  && sampleRepeat.isActive() == false )
      {
        sampleRepeat.activate();
      }
      else if ( key == 'f' || key == 'F' )
      {
        if ( filterOn == false )
        {
          println(" Activating Low Pass Sweep!");
          lowPassCutoff.activate( lowPassCutoffSweepLength, lowPassCutoff.getLastValues()[0], lowPassCutoffLo );
          filterOn = true;
        }
      }
      else if ( key == 'g' || key == 'G' )
      {
        if ( granulateBypass.isActive()  )
        {
          granulateBypass.deactivate();
        }
      }
    }
  }
  
  public void keyReleased()
  {
    char key = boombox.this.key;
    if ( key != PApplet.CODED )
    {
      if ( key == 'd' || key == 'D' )
      {
        player.setXDir( 0 );
      }
      else if ( key == 'a' || key == 'A' )
      {
        player.setXDir( 0 );
      }
      else if ( key == 'w' || key == 'W' )
      {
        player.setYDir( 0 );
      }
      else if ( key == 's' || key == 'S' )
      {
        player.setYDir( 0 );
      }
      else if ( key == 'r' || key == 'R' )
      {
        sampleRepeat.deactivate();
      }
      else if ( key == 'f' || key == 'F' )
      {
        lowPassCutoff.activate( 0.2f, lowPassCutoff.getLastValues()[0], lowPassCutoffHi );
        filterOn = false;
      }
      else if ( key == 'g' || key == 'G' )
      {
        granulateBypass.activate();
      }
    }
  }
}
// all the jams that the player has collected
// this also handles the visualize representation
// which is a box at the top of the screen that slides in and out
class Inventory
{
  // all my jams
  private ArrayList<Jam> mJams;
  // the jam the mouse is currently over
  private Jam       mCurrentJam;

  private ArrayList<EffectPickup> mEffects;

  // how tall the inventory box is
  private float     mHeight = 50.f;
  // Y coord of the bottom of the box
  private float     mBottom = 0.f;

  // where we started
  private float     mBegin = 0.f;
  // where we are headed to
  private float     mTarget = 0.f;
  // how long to take to go from begin to target
  private float     mLerpTime = 0.5f;
  // how much time has gone by in our lerp
  private float     mCurrTime;

  private float     mTapeScale = 1.f;


  // is the mouse inside of us?
  private boolean   mMouseInside;

  Inventory()
  {
    mJams = new ArrayList<Jam>();
    mEffects = new ArrayList<EffectPickup>();
    mCurrentJam = null;
    mCurrTime = mLerpTime;
    open();
  }
  
  public boolean isFull()
  {
    return mJams.size() == allJams.size() - 1 || key == 'C';
  }

  public void update( float dt )
  {
    if ( mCurrTime != mLerpTime )
    {
      mBottom = map( mCurrTime, 0, mLerpTime, mBegin, mTarget );
      mCurrTime += dt;
      if ( mCurrTime >= mLerpTime )
      {
        mCurrTime = mLerpTime;
        mBottom = mTarget;
      }
    }
    
    for(int i = 0; i < mJams.size(); ++i)
    {
      mJams.get(i).update( dt );
    }

    boolean mouseWasIn = mMouseInside;
    mMouseInside = mouseY < mBottom + 15.f;

    mCurrentJam = null;
    if ( mMouseInside )
    { 
      Iterator<Jam> iter = mJams.iterator();
      while ( iter.hasNext() )
      {
        Jam j = iter.next();
        if ( j.pointInside( mouseX, mouseY - mBottom ) )
        {
          mCurrentJam = j;
          break;
        }
      }

      if ( mCurrentJam != null )
      {
        if ( mCurrentJam.isPlaying() )
        {
          mouse.setState( Mouse.EJECT );
        }
        else if ( mCurrentJam.willPlay() == false )
        {
          mouse.setState( Mouse.PLAY );
        }
        else
        {
          mouse.setState( Mouse.EMPTY );
        }
      }
      else
      {
        mouse.setState( Mouse.EMPTY );
      }
    }

    //    if ( mouseWasIn && !mMouseInside )
    //    {
    //      close();
    //    }
    //    else if ( !mouseWasIn && mMouseInside )
    //    {
    //      open();
    //    }
  } 

  public boolean isActive()
  {
    return mMouseInside;
  }

  public void mouseMoved()
  {
  }

  public void mousePressed()
  {
    if ( mCurrentJam != null )
    {
      if ( mCurrentJam.isPlaying() || mCurrentJam.willPlay() )
      {
        player.eject();
      }
      else if ( mCurrentJam.willPlay() == false )
      {
        player.play();
      }

      mCurrentJam.queue();
    }
    else
    {
      for(int i = 0; i < mEffects.size(); ++i)
      {
        EffectPickup e = mEffects.get(i);
        println("Testing effect " + i);
        if ( e.pointInside( mouseX, mouseY - mBottom ) )
        {
          if ( e.isActive() == false )
          {
            println("Activating effect.");
            e.activate();
          }
          else
          {
            println("Deactivating effect.");
            e.deactivate();
          }
        }
        else
        {
          println("Point wasn't inside.");
        }
      }
    }
  }

  public void open()
  {
    mBegin = mBottom;
    mTarget = mHeight;
    mCurrTime = mLerpTime - mCurrTime;
  }

  public void close()
  {
    mBegin = mBottom;
    mTarget = 0.f;
    mCurrTime = mLerpTime - mCurrTime;
  }

  public void drawGroup( ArrayList<Jam> group )
  {
    int groupSize = group.size();
    // println("Drawing a group of " + groupSize + " jams.");
    if ( groupSize > 0 )
    {
      fill( 100 );
      noStroke();
      rectMode( CORNERS );

      Jam first = group.get(0);
      Jam last = group.get( groupSize - 1 );
      float horizPad = 5;
      float vertPad = 6;
      float minX = first.getPos().x - first.getWidth() * 0.5f - horizPad;
      float minY = first.getPos().y - first.getHeight() * 0.5f - vertPad;
      float maxX = last.getPos().x + last.getWidth() * 0.5f + horizPad;
      float maxY = last.getPos().y + last.getHeight() * 0.5f + vertPad;

      rect( minX, minY, maxX, maxY );

      for(int i = 0; i < groupSize; i++)
      {
        group.get(i).draw();
      }

      group.clear();
    }
  }

  public void drawEffectGroup( ArrayList<EffectPickup> group )
  {
    int groupSize = group.size();
    // println("Drawing a group of " + groupSize + " jams.");
    if ( groupSize > 0 )
    {
      fill( 100 );
      noStroke();
      rectMode( CORNERS );

      EffectPickup first = group.get(0);
      EffectPickup last = group.get( groupSize - 1 );
      float horizPad = 5;
      float vertPad = 6;
      float minX = first.getPos().x - first.getWidth() * 0.5f - horizPad;
      float minY = first.getPos().y - first.getHeight() * 0.5f - vertPad;
      float maxX = last.getPos().x + last.getWidth() * 0.5f + horizPad;
      float maxY = last.getPos().y + last.getHeight() * 0.5f + vertPad;

      rect( minX, minY, maxX, maxY );

      for(int i = 0; i < groupSize; i++)
      {
        group.get(i).draw();
      }
    }
  }

  public void draw()
  {
    pushMatrix();
    {
      // get into position
      translate( 0, mBottom );

      // background
      noStroke();
      fill( 67, 67, 67 );
      rectMode( CORNERS );
      rect( 0, -mHeight, width, 0 );

      int offset = 30;    
      // jams, guard against concurrent modification
      Jam[] jams = mJams.toArray( new Jam[] {
      } 
      );
      ArrayList<Jam> group = new ArrayList<Jam>();
      Jam prev = null;
      for( int i = 0; i < jams.length; i++)
      {
        Jam j = jams[i];

        // draw a box?
        if ( prev != null && prev.getCategory() != j.getCategory() )
        {
          drawGroup( group );
          offset += 12;
        }

        j.setPos( offset, -mHeight / 2 );
        group.add( j );

        offset += (int)j.getWidth() + 4;
        prev = j;
      }

      drawGroup( group );

      offset += 4;

      EffectPickup[] effects = mEffects.toArray( new EffectPickup[] {
      } 
      );
      for( int i = 0; i < effects.length; ++i)
      {
        EffectPickup e = effects[i];

        e.setPos( offset, -mHeight / 2 );
        offset += (int)e.getWidth() * 1.4f;
      }

      drawEffectGroup( mEffects );

      // dividing line
      stroke( 255, 255, 255 );
      strokeWeight( 2 );
      line( 0, 0, width, 0 );
    }
    popMatrix();
  }

  public void addJam( Jam j )
  {
    j.setScale( mTapeScale );
    mJams.add( j );
    Collections.sort( mJams );
  }

  public void addEffect( EffectPickup e )
  {
    mEffects.add( e );
  }
}

// the visual appearance for a Jam

// shared image for all jams
PImage TAPE = null;


public float getTapeWidth()
{
  return TAPE.width;
}

public float getTapeHeight()
{
  return TAPE.height;
}

class JamImage
{
  private int mTint;
  private PImage mTape;
  private AnimationInstance mTeethTurn;
  private AnimationInstance mTeethStill;
  private AnimationInstance mCurrentAnim;

  JamImage( int c )
  {
    mTint = c;
    mTape = TAPE;
    mTeethTurn = animationSystem.createAnimationInstance( "tapeTeethTurn" );
    mTeethStill = animationSystem.createAnimationInstance( "tapeTeethStill" );
    
    mCurrentAnim = mTeethStill;
  }

  public int getColor()
  {
    return mTint;
  }
  
  public void update( float dt )
  {
    mCurrentAnim.advance( dt );
  }
  
  public void play()
  {
    mCurrentAnim = mTeethTurn;
  }
  
  public void pause()
  {
    mCurrentAnim = mTeethStill;
  }

  public void draw( float x, float y, float s )
  {
    imageMode( CENTER );
    tint( mTint );
    pushMatrix();
      translate(x,y);
      scale(s);
      image( TAPE, 0, 0, getTapeWidth(), getTapeHeight() );
      tint( 220 );
      mCurrentAnim.draw();
    popMatrix();
  }
}



// a Jam is a cassette tape that the player can collect and then choose to play
class Jam extends UGen 
          implements Comparable<Jam>
{
  private PVector mPos;
  private float   mScale;
  private JamImage mImage;
  private String  mName;
  private float[] mLeftChannel;
  private float[] mRightChannel;
  private int     mSampleNum;
  private boolean mPaused;
  private Rectangle mCollision;
  
  private JamCategory mCategory;

  Jam( String jamName, JamCategory category, int c, float x, float y )
  {
    mPos = new PVector( x, y );
    mScale = 1.f;
    mImage = new JamImage(c);
    mName = jamName;
    mCategory = category;
    
    // extract uncompressed audio
    AudioSample sample = minim.loadSample( jamName );
    mLeftChannel = sample.getChannel( BufferedAudio.LEFT );
    mRightChannel = sample.getChannel( BufferedAudio.RIGHT );
    mSampleNum = 0;
    mPaused = true;
  }
  
  public int compareTo( Jam j )
  {
    return this.mCategory.compareTo( j.mCategory );
  }
  
  public JamCategory getCategory()
  {
    return mCategory;
  }

  public JamImage getImage()
  {
    return mImage;
  }

  public int getColor()
  {
    return mImage.getColor();
  }
  
  public void setSampleNum( int n )
  {
    mSampleNum = n;
  }
  
  public void setPaused( boolean paused )
  {
    mPaused = paused;
  }
  
  public void uGenerate( float[] out )
  {
    if ( mPaused == false )
    {
      out[0] = mLeftChannel[mSampleNum];
      out[1] = mRightChannel[mSampleNum];
      mSampleNum++;
      if ( mSampleNum == mLeftChannel.length )
      {
        mSampleNum = 0;
      }
    }
    else
    {
      out[0] = 0;
      out[1] = 0;
    }
  }

  public void setPos( float x, float y )
  {
    mPos.set( x, y, 0 );
  }

  public PVector getPos()
  {
    return mPos;
  }
  
  public void setScale( float s )
  {
    mScale = s;
  }
  
  public Rectangle getCollisionRectangle()
  {
    return new Rectangle( mPos.x, mPos.y, getWidth(), getHeight(), LEFT, TOP ); 
  }

  public float getWidth()
  {
    return getTapeWidth() * mScale;
  }
  
  public float getHeight()
  {
    return getTapeHeight() * mScale;
  }

  public boolean pointInside( float x, float y )
  {
    float hw = getWidth() * 0.5f;
    float hh = getHeight() * 0.5f;
    return ( x > mPos.x - hw && x < mPos.x + hw && y > mPos.y - hh && y < mPos.y + hh );
  }
  
  public void update( float dt )
  {
    if ( isPlaying() )
    {
      mImage.play();
    }
    else
    {
      mImage.pause();
    }
    mImage.update( dt );
  }
  
  public void drawOnlyImage()
  {
    mImage.draw( mPos.x, mPos.y, mScale );
  }

  public void draw()
  {
    mImage.draw( mPos.x, mPos.y, mScale );

    strokeWeight(2);
    stroke(255,255,255);
    if ( willPlay() )
    {
      strokeWeight(1);
      fill(0);
      float scount = jamSyncer.getSampleCount() + (jamSyncer.getMeasureCount() % 4) * jamSyncer.getLength();
      float totalSamp = jamSyncer.getLength() * 4;
      // println("scount = " + scount + " and totalSamp is " + totalSamp);
      float angle = map( scount, 0, totalSamp, 0, radians(360) );
      arc( mPos.x, mPos.y, 20, 20, -PI/2, -PI/2 + angle );
      strokeWeight(2);
      line( mPos.x - 5, mPos.y, mPos.x + 5, mPos.y );
      line( mPos.x, mPos.y + 5, mPos.x, mPos.y - 5 );
    }
    else if ( willEject() )
    {
      line( mPos.x - 5, mPos.y, mPos.x + 5, mPos.y );
    }
//    else if ( isPlaying() )
//    {
//      fill(0);
//      rectMode( CENTER );
//      rect( mPos.x, mPos.y, 10, 10 );
//    }
  }

  public boolean willPlay()
  {
    return jamSyncer.willJamPlay( this );
  }

  public boolean willEject()
  {
    return jamSyncer.willJamEject( this );
  } 

  public boolean isPlaying()
  {
    return jamSyncer.isJamPlaying( this );
  }

  public void queue()
  {
    jamSyncer.queueJam( this );
  }
}

// an interface to implement if you want to get called when the loop begins
interface LoopListener
{
  public void looped();
}

// a UGen to sync all of the Jams
class JamSyncer extends UGen
{
  // a list of AudioRecordingStreams that we are currently pulling from
  private ArrayList<Jam> mActive;
  // a list of AudioRecordingStream we should remove at the end of a loop
  private ArrayList<Jam> mToRemove;
  // a list of AudioRecordingStreams we should add at the end of a loop
  private ArrayList<Jam> mToAdd;
  // a dummy summer to bump the output count of active jams
  // eventually I should switch to using this instead of mActive
  private Summer mSink;
  
  // all of our loop listeners
  private ArrayList mLoopListeners;
  
  // where we are at in the sample count for one measure
  private int       mSampleCount;
  // how many samples are in one measure
  private int       mSamplesInMeasure;
  // how many measure have gone by
  private int       mMeasureCount;
  // what BPM we are syncing at
  private float    mBPM;
  
  JamSyncer( float bpm )
  {
    mActive = new ArrayList<Jam>();
    mToRemove = new ArrayList<Jam>();
    mToAdd = new ArrayList<Jam>();
    mLoopListeners = new ArrayList();
    mSink = new Summer();
    
    mSampleCount = 0;
    mMeasureCount = 0;
    mSamplesInMeasure = 0;
    mBPM = bpm;
  }
  
  public void addLoopListener( LoopListener ll )
  {
    mLoopListeners.add( ll );
  }
  
  public void removeLoopListener( LoopListener ll )
  {
    mLoopListeners.remove( ll );
  }
  
  public boolean willJamPlay( Jam jam )
  {
    return mToAdd.contains( jam );
  }
  
  public boolean willJamEject( Jam jam )
  {
    return mToRemove.contains( jam );
  }
  
  public boolean isJamPlaying( Jam jam )
  {
    return mActive.contains(jam);
  }
  
  public int getSampleCount()
  {
    return mSampleCount;
  }
  
  public int getLength()
  {
    return mSamplesInMeasure;
  }
  
  public int getMeasureCount()
  {
    return mMeasureCount;
  }
  
  public void queueJam( Jam jam )
  {
    boolean bActive = mActive.contains(jam);
    if ( bActive == false )
    {
      if ( mToAdd.contains(jam)  )
      {
        mToAdd.remove(jam);
        tapeStop.trigger();
      }
      else
      {
        // println("Adding a jam that is about " + jam.getMetaData().sampleFrameCount() + " samples long. We calc'd one loop at " + mSamplesInMeasure);
        mToAdd.add( jam );
        tapePlay.trigger();
      }
    }
    else if ( bActive == true && mToRemove.contains(jam) == false )
    {
      mToRemove.add( jam );
      tapeStop.trigger();
    }
  }
  
  private void calcSamplesInMeasure()
  {
    float beatPerSec = 60.f / mBPM;
    // 4 beats, one full measure
    float totalSamples = 4 * sampleRate() * beatPerSec;
    println("calcSamplesInMeasure: totalSamples = " + totalSamples);
    mSamplesInMeasure = (int)totalSamples;
  }
  
  public void setBPM( float bpm )
  {
    mBPM = bpm;
    calcSamplesInMeasure();
  }
  
  protected void sampleRateChanged()
  {
    calcSamplesInMeasure();
  }
  
  public void playJam( Jam jam )
  {
    mActive.add( jam );
    jam.setSampleNum( 0 );
    jam.setPaused( false );
    jam.patch( mSink );
  }
  
  private void removeJam( Jam j )
  {
      j.unpatch( mSink );
      j.setPaused( true );
      mActive.remove( j );
  }
  
  protected void uGenerate( float[] channels )
  {
    mSampleCount++;
    
    for(int i = 0; i < channels.length; ++i)
    {
      channels[i] = 0;
    }
    
    float[] samps = new float[channels.length];
    
    for(int i = 0; i < mActive.size(); ++i)
    {
      Jam j = mActive.get(i);
      j.tick( samps );
      for(int s = 0; s < samps.length; ++s)
      {
        channels[s] += samps[s];
      }
    }
    
    if ( mSampleCount == mSamplesInMeasure )
    {
      // remove elements from our active list that are in our toBeRemoved list
      for( int i = 0; i < mToRemove.size(); ++i )
      {
        Jam j = mToRemove.get(i);
        removeJam( j );
      }
      mToRemove.clear();
      
      mSampleCount = 0;
      mMeasureCount++;
      
      if ( mMeasureCount % 4 == 0 )
      {
        for(int i = 0; i < mToAdd.size(); ++i)
        {
          Jam j = mToAdd.get(i);
          
          // see if there is currently a jam of the same catergory playing
          // and remove it if the category is an exclusive one.
          if ( j.getCategory().isExclusive() )
          {
            Iterator<Jam> iter = mActive.iterator();
            while( iter.hasNext() )
            {
              Jam comp = iter.next();
              if ( comp.getCategory() == j.getCategory() )
              {
                removeJam( comp );
                break;
              }
            }
          }
          
          playJam( j );
        }
        mToAdd.clear();
        
        for( int i = 0; i < mActive.size(); ++i )
        {
          Jam j = mActive.get(i);
          j.setSampleNum( 0 );
        }
        
        // reset the granulate effect to keep it in sync as best as possiblefff
        granulate.reset();
        
        for(int i = 0; i < mLoopListeners.size(); ++i)
        {
          LoopListener ll = (LoopListener)mLoopListeners.get(i);
          ll.looped();
        }
      }
    }
  }
  
}

// visual appearance handler for the mouse
class Mouse
{
  // when there is nothing to click on
  static final int EMPTY = 0;
  // when you can click on a dude
  static final int JAM = 1;
  // when you are over a jam in your inventory
  static final int PLAY = 2;
  static final int EJECT = 3;
  
  private int mState;
  
  Mouse()
  {
    mState = EMPTY;
  }
  
  public void setState( int state )
  {
    mState = state;
  }
  
  public void draw()
  {
    
    switch( mState )
    {
    case EMPTY:
      {
        noFill();
        stroke( 56, 214, 212 );
        strokeWeight(1.2f);   
        ellipseMode( CENTER );
        ellipse( mouseX, mouseY, 8, 8 );  
      }
      break;
      
    case JAM:
      {
        tint(255);
        image( TAPE, mouseX, mouseY, getTapeWidth() * 0.5f, getTapeHeight() * 0.5f );
      }
      break;
    
    case PLAY:
      {
        fill( 56, 214, 212 );
        noStroke();
        triangle( mouseX - 5, mouseY - 5, mouseX - 5, mouseY + 5, mouseX + 5, mouseY );
      }
      break;
      
    case EJECT:
      {
        fill( 56, 214, 212 );
        noStroke();
        rectMode(CENTER);
        rect( mouseX, mouseY, 10, 10 );
      }
      break;
    }
    
  }
  
}
// helper class for drawing boxes and querying if a point is inside.
class Rectangle
{
    
  class Bounds
  {
    float minX, minY, maxX, maxY;
    
    Bounds( float x1, float y1, float x2, float y2 )
    {
      minX = x1;
      minY = y1;
      maxX = x2;
      maxY = y2;
    }
    
    public boolean pointInside( float x, float y )
    {
      return ( x >= minX && x <= maxX && y >= minY && y <= maxY );
    }
    
    public boolean collidesWith( Bounds otherBounds )
    {
      if ( maxY < otherBounds.minY ) return false;
      if ( minY > otherBounds.maxY ) return false;

      if ( maxX < otherBounds.minX ) return false;
      if ( minX > otherBounds.maxX ) return false;

      return true;
    }
    
    public void constrain( PVector pos )
    {
      pos.x = PApplet.constrain( pos.x, minX, maxX );
      pos.y = PApplet.constrain( pos.y, minY, maxY );
    }
  }
  
  
  private int mHorzAlign, mVertAlign;
  private float mWidth, mHeight;
  private float mX, mY;
  private Bounds mBounds;
  
  ArrayList<Rectangle> AdjacentRects;

  
  Rectangle( float x, float y, float w, float h, int ha, int va )
  {
    mX = x;
    mY = y;
    mWidth = w;
    mHeight = h;
    mHorzAlign = ha;
    mVertAlign = va;
    AdjacentRects = new ArrayList<Rectangle>();
    mBounds = new Bounds(0,0,0,0);
    calcBounds();
  }
  
  public void draw()
  {
    rectMode(CORNERS);
    rect( mBounds.minX, mBounds.minY, mBounds.maxX, mBounds.maxY );
  }
  
  public PVector getPos()
  {
    return new PVector(mX, mY);
  }
  
  public void setPos( float x, float y )
  {
    mX = x;
    mY = y;
    calcBounds();
  }
  
  public Bounds getBounds()
  {
    return mBounds;
  }
  
  public void constrain( PVector pos )
  {
    mBounds.constrain( pos );
  }
  
  public boolean pointInside( PVector pos )
  {
    return pointInside( pos.x, pos.y );
  }
  
  public boolean pointInside( float x, float y )
  {
    return mBounds.pointInside( x, y );
  }
  
  public boolean collidesWith( Rectangle otherRect )
  {
    return mBounds.collidesWith( otherRect.mBounds );
  }
  
  public void calcBounds()
  {
        // assume LEFT / TOP
    mBounds.minX = mX;
    mBounds.minY = mY;
    mBounds.maxX = mX + mWidth;
    mBounds.maxY = mY + mHeight;
    
    switch( mHorzAlign )
    {
      case CENTER: 
      {
        mBounds.minX -= mWidth * 0.5f; 
        mBounds.maxX -= mWidth * 0.5f; 
        break;
      }
      
      case RIGHT: 
      {
        mBounds.minX -= mWidth; 
        mBounds.maxX -= mWidth; 
        break;
      }
    }
    
    switch( mVertAlign )
    {
      case CENTER: 
      {
        mBounds.minY -= mHeight * 0.5f; 
        mBounds.maxY -= mHeight * 0.5f; 
        break;
      }
      
      case BOTTOM: 
      {
        mBounds.minY -= mHeight; 
        mBounds.maxY -= mHeight; 
        break;
      }
    }
  }
}
// a class that will sample and then repeat a section of incoming audio
// while it continues to tick the audio coming into it.
// it is a similar effect to Live's BeatRepeat
public class SampleAndRepeat extends UGen
{
  UGenInput audio;
  
  // the buffer of audio we will record into and then repeat
  private float[] m_buffer;
  // how long is our sample buffer
  private int m_sampleLength;
  // where are we recording/playing back in our buffer?
  private int m_currSample;
  
  private float m_tempo;
  private float m_sampleLengthInBeats;
  
  private static final int PASS = -1;
  private static final int SAMPLE = 0;
  private static final int REPEAT = 1;
  
  private int m_state;
  
  public SampleAndRepeat( float tempo, float sampleLengthInBeats )
  {
    audio = new UGenInput();
    m_buffer = null;
    m_tempo = tempo;
    m_sampleLengthInBeats = sampleLengthInBeats;
    m_state = PASS;
  }
  
  public void setSampleLength( float tempo, float sampleLengthInBeats )
  {
    m_tempo = tempo;
    m_sampleLengthInBeats = sampleLengthInBeats;
    float beatPerSec = 60.f / tempo;
    m_sampleLength = PApplet.parseInt(sampleLengthInBeats * sampleRate() * beatPerSec);
  }
  
  public void sampleRateChanged()
  {
    setSampleLength( m_tempo, m_sampleLengthInBeats );
  }
  
  public boolean isActive() 
  {
    return ( m_state != PASS );
  }
  
  public void activate()
  {
    m_state = SAMPLE;
    m_currSample = 0;
  }
  
  public void deactivate()
  {
    m_state = PASS;
  }
  
  public void uGenerate( float[] channels )
  {
    // reinitialize buffer if things change
    if ( m_buffer == null || m_buffer.length != m_sampleLength * channels.length )
    {
      m_buffer = new float[ m_sampleLength * channels.length ];
    }
        
    switch( m_state )
    {
    case PASS:
      {
        arraycopy( audio.getLastValues(), channels );
      }
      break;
      
    case SAMPLE:
      {
        // pass the audio
        arraycopy( audio.getLastValues(), channels );
        // but also record it
        int offset = m_currSample * channels.length;
        arraycopy( channels, 0, m_buffer, offset, channels.length );
        float fadeIn = constrain( map( m_currSample, 0, 128, 0, 1 ), 0, 1 );
        float fadeOut = constrain( map( m_currSample, m_sampleLength - 128, m_sampleLength, 1, 0 ), 0, 1 );
        for(int i = offset; i < offset + channels.length; i++)
        {
          m_buffer[i] *= fadeIn;
          m_buffer[i] *= fadeOut;
        }
        
        m_currSample++;
        if ( m_currSample == m_sampleLength )
        {
          m_state = REPEAT;
          m_currSample = 0;
        }
      }
      break;
      
    case REPEAT:
      {
        arraycopy( m_buffer, m_currSample * channels.length, channels, 0, channels.length );
        m_currSample = ( m_currSample + 1 ) % m_sampleLength;
      }
      break;
    }
  }
}
PImage speakerBox;
PImage speakerConeTop;
PImage speakerConeBottom;

public void loadSpeakerImages()
{
  speakerBox = loadImage("SpeakerBox.png");
  speakerConeTop = loadImage("SpeakerTopCone.png");
  //speakerConeTop = loadImage("TopSpeaker.gif");
  //speakerConeBottom = loadImage("SpeakerBottomCone2.png");
  speakerConeBottom = loadImage("BottomSpeaker.gif");
  //speakerConeBottom = loadImage("bottomspeakergrey.png");
}

class Speaker
{
  PVector mPos;
  float   mScale;
  
  Speaker( float x, float y, float s )
  {
    mPos = new PVector(x,y);
    mScale = s;
  }
  
  public void draw()
  {
    pushMatrix();
    {
      translate( mPos.x, mPos.y );
      scale( mScale );
      noTint(); 
      imageMode( CENTER );
      image( speakerBox, 0, -speakerBox.height / 2.f, speakerBox.width, speakerBox.height );
      float coneScale = 1.f + mainOut.mix.level() * 0.8f;
      image( speakerConeTop, 0, -speakerBox.height * 0.72f, speakerConeTop.width * coneScale, speakerConeTop.height * coneScale );
      image( speakerConeBottom, 0, -speakerBox.height * 0.3f, speakerConeBottom.width * coneScale, speakerConeBottom.height * coneScale );
    } 
    popMatrix();
  }
  
  public void drawShadow()
  {
    pushMatrix();
    {
      translate( mPos.x, mPos.y );
      scale( mScale );
      
      noStroke();
      fill(0, 64);
      beginShape(QUADS);
        float xLeft = -speakerBox.width * 0.5f;
        float xSlant = -speakerBox.width * 0.8f;
        float xRight = speakerBox.width * 0.5f;
        float yHigh = speakerBox.height * 0.4f;
        vertex( xLeft, 0 );
        vertex( xLeft + xSlant, yHigh );
        vertex( xRight + xSlant, yHigh );
        vertex( xRight, 0 );
      endShape();
        
    } 
    popMatrix();
  }
}
// the graphics for the environment
class Stage
{ 
  Rectangle theGround;
  Rectangle moreGround;

  PImage theSky;
  float  mSkyAlpha = 0;
  
  PImage groundTex;

  // all the elevators in the level
  ArrayList<Elevator> mElevators;
  ArrayList<Speaker> mSpeakers;
  ArrayList<Rectangle> mPlatforms;
  ArrayList<Dude> mDudes;
  // I'm calling it collisions, but it's really just all walkable space
  // this includes the ground, platforms, and elevators
  ArrayList<Rectangle> mCollision; 

  // the collision box the player is currently inside of.
  Rectangle currCollision;

  FFT mFFT;

  float horizonHeight = height * 0.7f;

  Stage()
  {
    mFFT = new FFT( mainOut.bufferSize(), mainOut.sampleRate() );
    mFFT.logAverages( 11, 2 );

    theSky = loadImage("bg_01.png");
    groundTex = loadImage("ground_02.png");

    mElevators = new ArrayList<Elevator>();
    mPlatforms = new ArrayList<Rectangle>();
    mDudes = new ArrayList<Dude>();
    mCollision = new ArrayList<Rectangle>();
    mSpeakers = new ArrayList<Speaker>();

    float topOfGround = horizonHeight + 5;

    theGround = new Rectangle( 0, topOfGround + 8, 2150, height - horizonHeight - 15, LEFT, TOP );
    moreGround = new Rectangle( 2400, topOfGround + 8, 1500, height - horizonHeight - 15, LEFT, TOP );

    mCollision.add( theGround );
    mCollision.add( moreGround );

    currCollision = theGround;

    { 
      allJams.get(1).setPos( 230, 410 );

      Dude dude = new Dude( 410, 420, new Jam[] { 
        allJams.get(1)
      }
      , allJams.get(2) );
      mDudes.add( dude );

      // this goes on the platform
      allJams.get(3).setPos( 440, 150 );
      Rectangle platform = makePlatform( 400, 160, 460 );
      Elevator elevator = new Elevator( 910, topOfGround, topOfGround - 160, allJams.get(2) ); 

      connect( theGround, elevator );
      connect( elevator, platform );

      mElevators.add( elevator );

      mCollision.add( elevator.getRect() );

      mSpeakers.add( new Speaker( 455, topOfGround, 0.66f ) );
      mSpeakers.add( new Speaker( 805, topOfGround, 0.66f ) );
    }

    {
      Dude dudeGround = new Dude( 1030, 450, new Jam[] { allJams.get(3) }, allJams.get(4) );
      Dude dudePlatform = new Dude( 1500, 220, new Jam[] { allJams.get(2), allJams.get(4) }, allJams.get(6) );
      mDudes.add( dudeGround );
      mDudes.add( dudePlatform );

      allJams.get(5).setPos( 1500, 450 );

      Rectangle platform = makePlatform( 1200, 220, 400 );
      Elevator elevLeft = new Elevator( 1150, topOfGround, topOfGround - 220, allJams.get(4) );
      Elevator elevRight = new Elevator( 1650, topOfGround, topOfGround - 220, allJams.get(4) );

      connect( theGround, elevLeft );
      connect( elevLeft, platform );
      connect( platform, elevRight );
      connect( elevRight, theGround );

      mElevators.add( elevLeft );
      mElevators.add( elevRight );

      mCollision.add( elevLeft.getRect() );
      mCollision.add( elevRight.getRect() );

      mSpeakers.add( new Speaker( 1255, topOfGround, 0.41f ) );
      mSpeakers.add( new Speaker( 1545, topOfGround, 0.41f ) );
    }

    {
      allJams.get(11).setPos( 2000, 400 );

      Rectangle platform = makePlatform( 2100, 220, 350 );
      Elevator elevLeft = new Elevator( 2050, topOfGround, topOfGround - 220, allJams.get(5) );
      Elevator elevRight = new Elevator( 2500, topOfGround, topOfGround - 220, allJams.get(5) );

      connect( theGround, elevLeft );
      connect( elevLeft, platform );
      connect( platform, elevRight );
      connect( elevRight, moreGround );

      mElevators.add( elevLeft );
      mElevators.add( elevRight );

      mCollision.add( elevLeft.getRect() );
      mCollision.add( elevRight.getRect() );

      mSpeakers.add( new Speaker( 2145, topOfGround, 0.41f ) );
      mSpeakers.add( new Speaker( 2405, topOfGround, 0.41f ) );
    }

    {
      allJams.get(8).setPos( 2850, 400 );

      Dude dudeGround = new Dude( 2550, 470, new Jam[] { allJams.get(3), allJams.get(6), allJams.get(8) }, allJams.get(7) );
      Dude dudePlatform = new Dude( 3075, 225, new Jam[] { allJams.get(9), allJams.get(2), allJams.get(11), allJams.get(4), allJams.get(8) }, allJams.get(12) );

      mDudes.add( dudeGround );
      mDudes.add( dudePlatform );

      float leftPlatWidth = 2900 - 2675;
      Rectangle leftPlat = makePlatform( 2675, 125, leftPlatWidth );
      Rectangle rightPlat = makePlatform( 3000, 225, 150 );

      Elevator elevLow = new Elevator( 2625, topOfGround, topOfGround - 125, allJams.get(10) );
      Elevator elevHigh = new Elevator( 2950, 223, 102, allJams.get(7) );

      connect( moreGround, elevLow );
      connect( elevLow, leftPlat );
      connect( leftPlat, elevHigh );
      connect( elevHigh, rightPlat );

      mElevators.add( elevLow );
      mElevators.add( elevHigh );

      mCollision.add( elevLow.getRect() );
      mCollision.add( elevHigh.getRect() );

      mSpeakers.add( new Speaker( 2675 + leftPlatWidth * 0.5f, topOfGround, 0.8f ) );
      mSpeakers.add( new Speaker( 3075, topOfGround, 0.39f ) );
    }

    {
      allJams.get(9).setPos( 3750, 120 );

      Dude dude = new Dude( 3675, 450, new Jam[] { allJams.get(4), allJams.get(5), allJams.get(3) }, allJams.get(10) );

      mDudes.add( dude );

      Rectangle leftPlat = makePlatform( 3400, 225, 150 );
      Rectangle rightPlat = makePlatform( 3650, 125, 150 );

      Elevator elevLow = new Elevator( 3350, topOfGround, topOfGround - 225, allJams.get(1) );
      Elevator elevHigh = new Elevator( 3600, 225, 100, allJams.get(6) );

      connect( moreGround, elevLow );
      connect( elevLow, leftPlat );
      connect( leftPlat, elevHigh );
      connect( elevHigh, rightPlat );

      mElevators.add( elevLow );
      mElevators.add( elevHigh );

      mCollision.add( elevLow.getRect() );
      mCollision.add( elevHigh.getRect() );

      mSpeakers.add( new Speaker( 3425, topOfGround, 0.39f ) );
      mSpeakers.add( new Speaker( 3525, topOfGround, 0.39f ) );
      mSpeakers.add( new Speaker( 3725, topOfGround, 0.8f ) );
    }
  }

  public Rectangle makePlatform( float x, float y, float w )
  {
    Rectangle platform = new Rectangle( x, y, w, 8, LEFT, TOP );
    mPlatforms.add( platform );
    mCollision.add( platform );

    return platform;
  }

  private void connect( Rectangle a, Rectangle b )
  {
    a.AdjacentRects.add( b );
    b.AdjacentRects.add( a );
  }

  private void connect( Elevator e, Rectangle b )
  {
    connect( e.getRect(), b );
  }

  private void connect( Rectangle a, Elevator e )
  {
    connect( a, e.getRect() );
  }


  public void update( float dt )
  {
    if ( inventory.isFull() && mSkyAlpha < 255 )
    {
      mSkyAlpha += 64 * dt;
    }
    
    for(int i = 0; i < mElevators.size(); ++i)
    {
      mElevators.get(i).update( dt );
    }

    for(int i = 0; i < mDudes.size(); i++)
    {
      mDudes.get(i).update( dt );
    }
  }

  public void constrainMovement( PVector currPos, PVector desiredPos )
  {
    // figure out if the new position is on collision
    boolean onCollision = currCollision.pointInside( desiredPos );
    for(int i = 0; i < currCollision.AdjacentRects.size(); i++)
    {
      onCollision = onCollision || currCollision.AdjacentRects.get(i).pointInside( desiredPos );
    }

    // if we aren't on the collision constrain to a collision rect
    if ( onCollision == false )
    {
      for(int i = 0; i < currCollision.AdjacentRects.size(); i++)
      {
        Rectangle collision = currCollision.AdjacentRects.get(i);
        if ( collision.pointInside( currPos ) )
        {
          collision.constrain( desiredPos );
          currCollision = collision;
          return;
        }
      }

      currCollision.constrain( desiredPos );
    }
  }

  public void draw()
  {
    rectMode(CORNERS);

    // the sky
    pushMatrix();
    if ( inventory.isFull() )
    { 
      translate( gameplayScreen.cameraOffset, 0, 0 );
      tint(255, mSkyAlpha);
      imageMode(CORNER);
      image(theSky, 0, 0, width, height);
    }
    popMatrix();

    // waveform in the sky
    {
      stroke(255, 0, 162, 128);
      strokeWeight(2);
      float waveCenter = 200;
      float sampleWidth = 8;
      float sampleSpacing = 0;
      for(int i = 0; i < mainOut.bufferSize() - 1; ++i)
      {
        float x1 = i * (sampleWidth + sampleSpacing);
        float x2 = x1 + sampleWidth;
        float y1 = waveCenter - mainOut.mix.get(i) * 100;
        float y2 = waveCenter - mainOut.mix.get(i+1) * 100;
        line(x1, y1, x2, y2);
      }
    }

    rectMode(CENTER);

    // equalizer river
    //if ( false )
    {
      mFFT.forward( mainOut.mix );

      float startX = theGround.getBounds().maxX + 20;
      float endX = moreGround.getBounds().minX - 20;
      int totalAverages = 12;
      float totalWidth = endX - startX;
      float spacerTotal = 12;
      float waveWidth = (totalWidth - spacerTotal) / (totalAverages/2);
      float spacerSize = spacerTotal / (totalAverages/2);

      float minY = theGround.getBounds().minY + 20;

      // evens first
      int evens = totalAverages/2;
      for(int i = 0; i < evens; ++i )
      {
        float xOff = i * (waveWidth + spacerSize);

        float minX = startX + xOff;
        float maxX = minX + waveWidth;
        float topY = minY - mFFT.getAvg(i) * random(0.2f, 0.3f);
        float bottomY = height;

        noStroke();
        beginShape(QUADS);
          fill( 0, 0, 200 );
          vertex( minX, topY );
          fill( 0, 0, 100 );
          vertex( minX, bottomY );
          vertex( maxX, bottomY );
          fill( 0, 0, 200 );
          vertex( maxX, topY );
        endShape();

        stroke( 200 );
        strokeWeight( 2 );
        line( minX, topY, maxX, topY );
      }
      
      minY += 20;
      
      // odds on top
      int odds = totalAverages/2 - 1;
      for(int i = 0; i < odds; ++i )
      {
        float xOff = i * (waveWidth + spacerSize);

        float minX = startX + (waveWidth*0.5f) + xOff;
        float maxX = minX + waveWidth;
        float topY = minY - mFFT.getAvg(i + evens) * random(0.3f, 0.4f);
        float bottomY = height;

        noStroke();
        beginShape(QUADS);
          fill( 0, 0, 200 );
          vertex( minX, topY );
          fill( 0, 0, 100 );
          vertex( minX, bottomY );
          vertex( maxX, bottomY );
          fill( 0, 0, 200 );
          vertex( maxX, topY );
        endShape();

        stroke( 200 );
        strokeWeight( 2 );
        line( minX, topY, maxX, topY );
      }
      
      
      
    }

    rectMode(CORNERS);

    // the ground and horizon
    {
      //      noStroke();
      //      fill(105, 122, 144);
      //      rect( -10, horizonHeight, theGround.getBounds().maxX + 30, height + 10);
      //      
      Rectangle.Bounds moreBounds = moreGround.getBounds();
      //      rect( moreBounds.minX - 30, horizonHeight, moreBounds.maxX + 10, height + 10);

      noFill();
      noStroke();
      float lowY = horizonHeight;
      float hiY = horizonHeight + groundTex.height;
      beginShape(QUADS);
      texture( groundTex );
      textureMode( NORMALIZED );
      tint(105, 122, 144);
      // first ground
      for( int x = -10; x < theGround.getBounds().maxX + 30; x += groundTex.width )
      {
        vertex(x, lowY, 0, 0);
        vertex(x, hiY, 0, 1);
        vertex(x + groundTex.width, hiY, 1, 1);
        vertex(x + groundTex.width, lowY, 1, 0);
      }

      // second ground
      for( float x = moreGround.getBounds().minX - 30; x < moreGround.getBounds().maxX + 30; x += groundTex.width )
      {
        vertex(x, lowY, 0, 0);
        vertex(x, hiY, 0, 1);
        vertex(x + groundTex.width, hiY, 1, 1);
        vertex(x + groundTex.width, lowY, 1, 0);
      }

      endShape();

      // horizon line
      strokeWeight(2);
      stroke(0);
      line( -10, horizonHeight, theGround.getBounds().maxX + 34, horizonHeight );
      line( moreBounds.minX - 30, horizonHeight, moreBounds.maxX + 30, horizonHeight );
    }

    // the speakers
    {
      for(int i = 0; i < mSpeakers.size(); ++i)
      {
        mSpeakers.get(i).draw();
      }
    }

    // the platforms
    {

      for (int i = 0; i < mPlatforms.size(); i++)
      {
        Rectangle thePlatform = mPlatforms.get(i);
        Rectangle.Bounds plat = thePlatform.getBounds();
        float lowY = plat.minY;
        float hiY = plat.maxY + 10;
        float v = 22f;

        noStroke();
        tint(105, 122, 144);
//        fill(105, 122, 144);
//        rect( plat.minX, plat.minY, plat.maxX, plat.maxY + 10 );

        beginShape(QUADS);
        texture(groundTex);
        textureMode(IMAGE);
        for( float x = plat.minX; x < plat.maxX; x += groundTex.width )
        {
          float w = groundTex.width;
          if ( x + w > plat.maxX )
          {
            w = plat.maxX - x;
          }
          vertex(x, lowY, 0, 0);
          vertex(x, hiY, 0, v);
          vertex(x + w, hiY, w, v);
          vertex(x + w, lowY, w, 0);
        }
        endShape();

        // line across the top
        strokeWeight(2);
        stroke(179, 183, 188);
        line( plat.minX, plat.minY, plat.maxX, plat.minY );
      }
    }

    // sidewalk cracks
    {
      //      strokeWeight(2);
      //      stroke(60, 62, 64);
      //      float lineSpacing = 100;
      //      float lineSlope = 75;
      //      for(int i = 0; i < 20; ++i)
      //      {
      //        line( lineSlope*i, height, lineSpacing + lineSlope*i, horizonHeight );
      //      }
    }

    // ground walkable space
    if ( DRAW_COLLISION )
    {
      noStroke();
      fill(COLLISION_COLOR);
      theGround.draw();
    }

    // the platform walkable space
    if ( DRAW_COLLISION )
    {
      noStroke();
      fill(COLLISION_COLOR);
      for(int i = 0; i < mPlatforms.size(); i++)
      {
        mPlatforms.get(i).draw();
      }
    }

    for(int i = 0; i < mElevators.size(); i++)
    {
      mElevators.get(i).draw();
    }

    // sort DUDES and Player
    {
      ArrayList<Dude> dudesBehind = new ArrayList<Dude>();
      ArrayList<Dude> dudesInFront = new ArrayList<Dude>();
      for(int i = 0; i < mDudes.size(); i++)
      {
        if ( mDudes.get(i).getPos().y <= player.getPos().y )
        {
          dudesBehind.add( mDudes.get(i) );
        }
        else
        {
          dudesInFront.add( mDudes.get(i) );
        }
      }

      for(int i = 0; i < dudesBehind.size(); i++)
      {
        dudesBehind.get(i).draw();
      }
      player.draw();
      for(int i = 0; i < dudesInFront.size(); i++)
      {
        dudesInFront.get(i).draw();
      }
    }

    // finally shadow pass
    if ( false )
    {
      for(int i = 0; i < mSpeakers.size(); ++i)
      {
        mSpeakers.get(i).drawShadow();
      }
    }
  }
}

// stars that pop up when you jam with a dude

PImage dopeStar = null;
PImage freshStar = null;

public void loadStarImages()
{
 dopeStar = loadImage("StarDope.png");
 freshStar = loadImage("StarFresh.png"); 
}

class Star
{
  private PImage mStar;
  private PVector mPos;
  private float  mScale;
  
  private float s_startScale = 0.1f;
  private float s_endScale = 3.f;
  private float s_scaleRate = 3.f;
  
  Star(float x, float y)
  {
    if ( random(1) < 0.5f)
    {
      mStar = dopeStar;
    }
    else
    {
      mStar = freshStar;
    }
    mScale = s_startScale;
    mPos = new PVector(x, y);
  }
  
  public boolean update( float dt )
  {
    mScale += s_scaleRate * dt;
    if ( mScale > s_endScale )
    {
      return true;
    }
    
    return false;
  }
  
  public void draw()
  {
    float alph = map(mScale, s_startScale, s_endScale, 255, 0);
    tint(255, alph);
    imageMode(CENTER);
    pushMatrix();
      translate(mPos.x, mPos.y);
      scale(mScale);
      image(mStar, 0, 0);
    popMatrix();
  }
}
class TitleScreen
{
  PFont titleFont;
  boolean m_active;
  
  PImage titleImg;
  PImage loading;
  PImage clickPlay;
  
  TitleScreen()
  {
    titleFont = loadFont("ComicSansMS-48.vlw");
    m_active = true;
    
    titleImg = loadImage("Title.png");
    loading = loadImage("Loading.png");
    clickPlay = loadImage("ClickToPlay.png");
  }
  
  public boolean isActive()
  {
    return m_active;
  }
  
  public void draw()
  {
    imageMode(CORNER);
    
    image(titleImg, 0, 0);
    
    imageMode(CENTER);
    
    if ( loaderThread.isAlive() )
    {
      image(loading, width/2, height/2 + 50);
    }
    else
    {
      image(clickPlay, width/2, height/2 + 50);
    }
  }
  
  public void mousePressed()
  {
    if ( loaderThread.isAlive() == false )
    {
      m_active = false;
      jamSyncer.playJam( allJams.get(0) );
      jamSyncer.patch( granulateBypass ).patch( sampleRepeat ).patch( lowPass ).patch( globalGain ).patch( mainOut );
    }
  }
}
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "boombox" });
  }
}
